@@ -83,10 +83,9 @@ proc mm_to_voxels { X Y Z mask } {
     upvar $Y cY
     upvar $Z cZ
 
-
-    set vcX [ exec sh -c "echo $cX $cY $cZ | $FSLDIR/bin/std2imgcoord -img $mask -vox - | awk '{print \$1}'" ]    
-    set vcY [ exec sh -c "echo $cX $cY $cZ | $FSLDIR/bin/std2imgcoord -img $mask -vox - | awk '{print \$2}'" ] 
-    set vcZ [ exec sh -c "echo $cX $cY $cZ | $FSLDIR/bin/std2imgcoord -img $mask -vox - | awk '{print \$3}'" ] 	
+    set vcX [ exec sh -c "echo $cX $cY $cZ | $FSLDIR/bin/std2imgcoord -img $mask -vox - 2>/dev/null | awk '{print \$1}'" ]    
+    set vcY [ exec sh -c "echo $cX $cY $cZ | $FSLDIR/bin/std2imgcoord -img $mask -vox - 2>/dev/null | awk '{print \$2}'" ] 
+    set vcZ [ exec sh -c "echo $cX $cY $cZ | $FSLDIR/bin/std2imgcoord -img $mask -vox - 2>/dev/null | awk '{print \$3}'" ] 
     set cX $vcX
     set cY $vcY
     set cZ $vcZ
@@ -301,8 +300,7 @@ proc fdt:dialog { w tclstartupfile } {
     FileEntry $w.data.seed.ssf.xfm -textvariable probtrack(xfm)  -label "Select Seed to diff transform" -title "Select seed-space to DTI-space transformation matrix" -filetypes *
     FileEntry $w.data.seed.ssf.reference -textvariable probtrack(reference) -label "Seed reference image:" -title "Choose reference image" -filetypes IMAGE 
     pack $w.data.seed.ssf.ssd -side top -anchor nw
-
-    if { [ file exists /usr/local/fsl/bin/reord_OM ] } {
+    if { [ file exists ${FSLDIR}/bin/reord_OM ] } {
 	frame  $w.data.seed.bcf
 	checkbutton $w.data.seed.bcf.bc -text "Blind Classification:" -variable probtrack(bcyn)  -command " pack forget $w.data.seed.bcf.w ; if { \$probtrack(bcyn) } { pack $w.data.seed.bcf.w  -side left} ; $w.probtrack compute_size"
 	set probtrack(scale) 5
@@ -520,7 +518,7 @@ proc fdt:dialog { w tclstartupfile } {
 }
 
 proc fdt:probtrack_mode { w } {
-    global probtrack
+    global probtrack FSLDIR
 
     pack forget $w.data.seed.voxel $w.data.seed.ssf  $w.data.seed.ssf.xfm $w.data.seed.ssf.reference $w.data.seed.bcf $w.data.seed.target $w.data.targets.cf
     $w.data.dir configure -label  "Output directory:" -title  "Name the output directory" -filetypes *
@@ -534,7 +532,7 @@ proc fdt:probtrack_mode { w } {
 	seedmask {
 	    pack $w.data.seed.ssf -in $w.data.seed.f -side bottom -anchor w -pady 2
             pack forget $w.data.seed.ssf.ssd
-	    if { [ file exists /usr/local/fsl/bin/reord_OM ] } {
+	    if { [ file exists ${FSLDIR}/bin/reord_OM ] } {
 		pack $w.data.seed.bcf -in $w.data.seed.f -side bottom -anchor w -pady 2
 	    }
 	    pack $w.data.targets.cf -in $w.data.targets.f -anchor w
@@ -815,15 +813,20 @@ proc fdt:apply { w dialog } {
                     set flags "--mode=simple --seedref=$probtrack(reference) -o $probtrack(output) -x ${filebase}_coordinates.txt $flags"
 	       } 
                seedmask {
-		   if { [ file exists /usr/local/fsl/bin/reord_OM ] } {
+		   if { [ file exists ${FSLDIR}/bin/reord_OM ] } {
 		       if { $probtrack(bcyn) } { 
-			   fdt_monitor_short $w "${FSLDIR}/bin/convert_xfm -omat $probtrack(output)/tmp_xfm_mat -inverse $probtrack(xfm)"
-			   fdt_monitor_short $w "${FSLDIR}/bin/flirt -in $probtrack(bedpost_dir)/nodif_brain_mask -ref $probtrack(reference) -applyxfm -init $probtrack(output)/tmp_xfm_mat -out $probtrack(output)/tmp_brain_mask"
+
+                           if { $probtrack(xfm) != "" } {
+			       fdt_monitor_short $w "${FSLDIR}/bin/convert_xfm -omat $probtrack(output)/tmp_xfm_mat -inverse $probtrack(xfm)"
+			       fdt_monitor_short $w "${FSLDIR}/bin/flirt -in $probtrack(bedpost_dir)/nodif_brain_mask -ref $probtrack(reference) -applyxfm -init $probtrack(output)/tmp_xfm_mat -out $probtrack(output)/tmp_brain_mask"
+			   } else {
+			       fdt_monitor_short $w "${FSLDIR}/bin/flirt -in $probtrack(bedpost_dir)/nodif_brain_mask -ref $probtrack(reference) -applyxfm -init ${FSLDIR}/etc/flirtsch/ident.mat -out $probtrack(output)/tmp_brain_mask"
+			   }
 			   fdt_monitor_short $w "${FSLDIR}/bin/flirt -in $probtrack(output)/tmp_brain_mask -ref $probtrack(output)/tmp_brain_mask -applyisoxfm $probtrack(scale) -out $probtrack(output)/lowresmask"
 			   fdt_monitor_short $w "${FSLDIR}/bin/fslmaths  $probtrack(output)/lowresmask -thr 0.5 -bin  $probtrack(output)/lowresmask"
 			   set flags "$flags --lrmask=$probtrack(output)/lowresmask --omatrix2" 
 			   fdt_monitor_short $w "${FSLDIR}/bin/imrm $probtrack(output)/tmp_brain_mask"
-			   fdt_monitor_short $w "/bin/rm $probtrack(output)/tmp_xfm_mat"
+			   if  { [ file exists $probtrack(output)/tmp_xfm_mat ] } { fdt_monitor_short $w "/bin/rm $probtrack(output)/tmp_xfm_mat" }
 		       }
 		   }
                    set flags "--mode=seedmask -x $probtrack(reference) $flags"  
@@ -851,7 +854,7 @@ proc fdt:apply { w dialog } {
 		    puts $log " $w.data.targets.cf.tf.targets insert end [  $w.data.targets.cf.tf.targets get 0 end ]"
                     set flags "$flags --targetmasks=$probtrack(output)/targets.txt --os2t "
                 }
-	    
+		close $log
 		if { $FSLPARALLEL } {
                     set script [open "${filebase}_script.sh" w]
                     puts "${filebase}_script.sh"
@@ -882,7 +885,6 @@ proc fdt:apply { w dialog } {
 		    puts "rm ${filebase}_coordinates.txt"
 		    exec rm ${filebase}_coordinates.txt
 		}
-		close $log
 		if { $copylog != "" } {
 		    puts "mv $logfile $copylog"
 		    exec mv $logfile $copylog
