@@ -64,7 +64,6 @@
 
 #include "ptx_meshmask.h"
 #include "streamlines.h"
-
 using namespace std;
 using namespace NEWIMAGE;
 using namespace TRACT;
@@ -84,7 +83,7 @@ void meshmask()
   mseeds.load_fs_label(opts.seedfile.value());
 
   // internally create seed mask in voxel space
-  volume<int> seeds;
+  volume<float> seeds;
   if(opts.seedref.value()!="")
     read_volume(seeds,opts.seedref.value());
   else
@@ -99,6 +98,11 @@ void meshmask()
   counter.initialise();
   Seedmanager seedmanager(counter);
 
+  Matrix mm_to_vox(4,4);
+  mm_to_vox << -1 << 0 << 0 <<  (seeds.xsize()-1)/2
+	    <<  0 << 0 << -1 << (seeds.zsize()-1)/2
+	    <<  0 << 1 << 0 <<  (seeds.ysize()-1)/2
+	    <<  0 << 0 << 0 << 1;
 
   ColumnVector fs_coord_mm(4),xyz_vox,seeddim(3);
   seeddim << seeds.xdim() << seeds.ydim() << seeds.zdim();
@@ -106,12 +110,15 @@ void meshmask()
   int keeptotal=0;
 
   for(vector<Mpoint*>::iterator i = mseeds._points.begin();i!=mseeds._points.end();i++){
-    if((*i)->get_value() > 0){
+    //if((*i)->get_value() > 0){
 
       fs_coord_mm<<(*i)->get_coord().X<<(*i)->get_coord().Y<<(*i)->get_coord().Z << 1.0; 
-      xyz_vox = seeds.qform_mat().i()*fs_coord_mm;
+      //      xyz_vox = seeds.qform_mat().i()*fs_coord_mm
+      xyz_vox = mm_to_vox*fs_coord_mm;
 
       float x=xyz_vox(1);float y=xyz_vox(2);float z=xyz_vox(3);
+      Pt newPt(x,y,z);
+                (*i)->_update_coord = newPt;
 
       seeds(round(x),round(y),round(z)) = 1;
     
@@ -119,9 +126,10 @@ void meshmask()
       dir << (*i)->local_normal().X << (*i)->local_normal().Y << (*i)->local_normal().Z;
       keeptotal += seedmanager.run(round(x),round(y),round(z),true,-1,dir); 
 	
-    }
+      //}
   }
-
+  mseeds.update();
+  mseeds.save("test.vtk",3);
   //return;
 
 //   for(int z=0;z<seeds.zsize();z++){
