@@ -68,7 +68,8 @@
 
 #include "utils/options.h"
 #include "miscmaths/miscmaths.h"
-#include "newimage/warpfns.h"
+#include "warpfns/warpfns.h"
+#include "warpfns/fnirt_file_reader.h"
 
 #define _GNU_SOURCE 1
 #define POSIX_SOURCE 1
@@ -124,8 +125,8 @@ Option<string> initwarpname(string("--initwarp"), string(""),
 			string("filename for initial warp transform (volume)"),
 			false, requires_argument);
 Option<string> refvolname(string("-r,--ref"), string(""),
-		       string("filename for reference image (determines inverse warpvol's FOV and pixdims)"),
-		       false, requires_argument);
+		       string("filename for new reference image, i.e., what was originally the input image (determines inverse warpvol's FOV and pixdims)"),
+		       true, requires_argument);
 Option<string> outname(string("-o,--out"), string(""),
 		       string("filename for output (inverse warped) image"),
 		       true, requires_argument);
@@ -292,9 +293,11 @@ Option<string> warpname(string("-w,--warp"), string(""),
     y1 = (*func)(x1*unitdir + pt);
     findinitialbound(x1,xmid,x2,y1,ymid,y2,func,unitdir,pt);
 
-    cout << "BOUND = (" << x1 << "," << y1 << ")  ";
-    cout << "(" << xmid << "," << ymid << ")  ";
-    cout << "(" << x2 << "," << y2 << ")" << endl;
+    if (verbose.value()) {
+      cout << "BOUND = (" << x1 << "," << y1 << ")  ";
+      cout << "(" << xmid << "," << ymid << ")  ";
+      cout << "(" << x2 << "," << y2 << ")" << endl;
+    }
 
     float min_dist = 0.1 * unittol;
     float xnew, ynew;
@@ -359,30 +362,7 @@ Option<string> warpname(string("-w,--warp"), string(""),
 
 int fill_image(volume4D<float>& invol, const volume<float>& mask)
 {
-  // fill each 3D image in invol with values generated by successive
-  //  mean dilation (starting with the mask and growing until all voxels
-  //  have been filled)
-  volume<float> kernel(3,3,3);
-  kernel=1.0f;
-
-  volume<float> fillmask(mask);
-  fillmask = binarise(fillmask,0.5f);
-  invol *= fillmask;
-  
-  while (fillmask.sum()<(fillmask.nvoxels()-0.5)) {
-    for (int n=invol.mint(); n<=invol.maxt(); n++) {
-      invol[n] = morphfilter(invol[n],kernel,string("dilateM"));
-    }
-    fillmask = morphfilter(fillmask,kernel,string("dilateM"));
-    for (int n=invol.mint(); n<=invol.maxt(); n++) {
-      invol[n] = divide(invol[n],fillmask,fillmask);
-    }
-    fillmask.binarise(0.001);
-    if (verbose.value()) {
-      cout << " Filled " << fillmask.sum() << " voxels out of " << fillmask.nvoxels() << endl;
-    }
-  }
-
+  invol = sparseinterpolate(invol,mask);
   return 0;
 }
 
@@ -448,6 +428,7 @@ float calc_cost(volume4D<float>& costim,
 		const volume4D<float>& warp, 
 		const volume4D<float>& invwarp, float lambda=0.0)
 {
+  // C = \sum (x_d - x_u)^2 / N = \sum c_x
   float cost=0.0;
   costim=0.0;
   for (int z=warp.minz(); z<=warp.maxz(); z++) {
@@ -627,40 +608,29 @@ int gradient_descent(volume4D<float>& invwarp, const volume4D<float>& warpvol, i
 
 int invwarp()
 {
+ 
+ // read in images
+  volume4D<float>    invwarp;
+  volumeinfo         vinfo;
+  volume4D<float>    warpvol;
+  AbsOrRelWarps      spec_wt=UnknownWarps;        // Specified warp convention
+  Matrix             skrutt=IdentityMatrix(4);    // Not used
 
-  // read in images
-  volume4D<float> invwarp;
-  volumeinfo vinfo;
-  volume4D<float> warpvol;
+  read_volume4D(warpvol,warpname.value());
 
-  read_volume4D(warpvol,warpname.value(),vinfo);
-
-  if (abswarp.set()) { abs_warp = true; }
-  else if (relwarp.set()) { abs_warp = false; }
-  else {
-    if (verbose.value()) { 
-      cout << "Automatically determining relative/absolute warp conventions" << endl; 
-    }
-    abs_warp = is_abs_convention(warpvol);
-    if (verbose.value()) {
-      if (abs_warp) { cout << "Warp convention = absolute" << endl; } 
-      else { cout << "Warp convention = relative" << endl; }
-    }
-  }
-
-  if (!abs_warp) {
-    convertwarp_rel2abs(warpvol);
-  }
+  if (abswarp.value()) spec_wt = AbsoluteWarps;
+  else if (relwarp.value()) spec_wt = RelativeWarps;
+  FnirtFileReader    fnirtfile(warpname.value(),spec_wt,verbose.set());
+  warpvol = fnirtfile.FieldAsNewimageVolume4D(true);
+  convertwarp_rel2abs(warpvol); 
 
   // read reference volume and set size of invwarp
-  if (refvolname.set()) { 
-    read_volume4D(invwarp,refvolname.value());
-    while (invwarp.tsize()<3) { invwarp.addvolume(invwarp[0]); }
-    while (invwarp.tsize()>3) { invwarp.deletevolume(invwarp.maxt()); }
-    // inwarp.tsize() == 3 here
-    invwarp=0.0f;
-    if (debug.value()) { print_volume_info(invwarp,"invwarp"); }
-  }
+  read_volume4D(invwarp,refvolname.value(),vinfo);
+  while (invwarp.tsize()<3) { invwarp.addvolume(invwarp[0]); }
+  while (invwarp.tsize()>3) { invwarp.deletevolume(invwarp.maxt()); }
+  // inwarp.tsize() == 3 here
+  invwarp=0.0f;
+  if (debug.value()) { print_volume_info(invwarp,"invwarp"); }
 
   // set up the initial warp field
   volume<float> mask;
@@ -673,9 +643,9 @@ int invwarp()
   if (debug.value()) { 
     print_volume_info(invwarp,"invwarp");
     if (outname.set()) {   // save results
-      if (!abs_warp) { convertwarp_abs2rel(invwarp); }
+      if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_abs2rel(invwarp); }
       save_volume4D(invwarp,fslbasename(outname.value())+"_init",vinfo);
-      if (!abs_warp) { convertwarp_rel2abs(invwarp); }
+      if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_rel2abs(invwarp); }
     }
   }
 
@@ -683,11 +653,23 @@ int invwarp()
   volume4D<float> deriv, costim(warpvol);
   if (verbose.value()) { cout << "Perform gradient descent" << endl; }
   gradient_descent(invwarp,warpvol,niter.value(),deriv,costim);
-  
+  if (verbose.value()) { cout << "After gradient descent" << endl; }
+
+  if (debug.value()) { 
+      if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_abs2rel(invwarp); }
+      save_volume4D(invwarp,fslbasename(outname.value())+"_prefill",vinfo);
+      if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_rel2abs(invwarp); }
+  }
+
   if (!nojaccon.value()) {
     if (verbose.value()) { cout << "Re-fill image using valid mask" << endl; }
     fill_warp(invwarp,mask);
     
+    if (debug.value()) { 
+      if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_abs2rel(invwarp); }
+      save_volume4D(invwarp,fslbasename(outname.value())+"_postfill",vinfo);
+      if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_rel2abs(invwarp); }
+    }
     if (verbose.value()) { cout << "Constrain Jacobian" << endl; }
     constrain_topology(invwarp,jmin.value(),jmax.value());
     
@@ -699,16 +681,16 @@ int invwarp()
   }
 
 
+  if (verbose.value()) { cout <<"Constrained Jacobian"<< endl; }
 
-  if (outname.set()) {
-    if (!abs_warp) { convertwarp_abs2rel(invwarp); }
-    save_volume4D(invwarp,outname.value(),vinfo);
-    if (debug.value()) {
-      save_volume4D(deriv,fslbasename(outname.value())+"_deriv",vinfo);
-      save_volume4D(costim,fslbasename(outname.value())+"_costim",vinfo);
-    }
+  if (fnirtfile.AbsOrRel()==RelativeWarps) { convertwarp_abs2rel(invwarp); }
+  save_volume4D(invwarp,outname.value(),vinfo);
+  if (debug.value()) {
+    save_volume4D(deriv,fslbasename(outname.value())+"_deriv",vinfo);
+    save_volume4D(costim,fslbasename(outname.value())+"_costim",vinfo);
   }
-  return 0;
+
+  return(EXIT_SUCCESS);
 }
 
 
@@ -737,14 +719,20 @@ int main(int argc, char *argv[])
     options.add(verbose);
     options.add(help);
     
-    options.parse_command_line(argc, argv);
+    int nparsed = options.parse_command_line(argc, argv);
+    if (nparsed < argc) {
+      for (; nparsed<argc; nparsed++) {
+        cerr << "Unknown input: " << argv[nparsed] << endl;
+      }
+      exit(EXIT_FAILURE);
+    }
 
     if ( (help.value()) || (!options.check_compulsory_arguments(true)) )
       {
 	options.usage();
 	exit(EXIT_FAILURE);
       }
-  }  catch(X_OptionError& e) {
+  } catch(X_OptionError& e) {
     options.usage();
     cerr << endl << e.what() << endl;
     exit(EXIT_FAILURE);
@@ -752,6 +740,27 @@ int main(int argc, char *argv[])
     cerr << e.what() << endl;
   } 
 
+  if (abswarp.value() && relwarp.value()) {
+    cerr << "--abs and --rel flags cannot both be set" << endl;
+    exit(EXIT_FAILURE);
+  }
+
+  volume4D<float>   warpvol;
+  try {
+    read_volume4D_hdr_only(warpvol,warpname.value());
+  }
+  catch(...) {
+    cerr << "invwarp: Problem reading warp-file " << warpname.value() << endl;
+    exit(EXIT_FAILURE);
+  }
+  if ((warpvol.intent_code()==FSL_CUBIC_SPLINE_COEFFICIENTS || 
+       warpvol.intent_code()==FSL_QUADRATIC_SPLINE_COEFFICIENTS || 
+       warpvol.intent_code()==FSL_DCT_COEFFICIENTS ||
+       warpvol.intent_code()==FSL_FNIRT_DISPLACEMENT_FIELD) &&
+      (abswarp.value() || relwarp.value())) {
+    cout << "--abs and --rel flags ignored when reading fnirt coefficient files" << endl;
+  }
+
   return invwarp();
 }
 
